//+------------------------------------------------------------------+
//|                                           EGEN Power Painter.mq4 |
//|                      Copyright  2011, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"

#define  D1 0
#define  H4 1
#define  H1 2

#define PERIOD.D1 1597
#define PERIOD.H4 233
#define PERIOD.H1 55

extern bool Signal.1=true;
extern bool Signal.2=true;

int POWER[PERIOD_D1],MAX[3],MIN[3],AVG;

bool EQ(int value1,int value2)
{
   return(value1==value2);
}

bool ZERO(int value)
{
   return(value==0);
}

bool MR(int value1,int value2)
{
   return(value1>value2);
}

bool NOT(bool flag)
{
   return(!flag);
}

bool BARS(int shift)
{
   return(shift<iBars(NULL,PERIOD_M1));
}

int SUM(double price)
{
   string P=DoubleToStr(price,Digits);
   int S=0;
   for(int i=0; i<StringLen(P); i++) S+=StrToInteger(StringSubstr(P,i,1));
   return(S);
}

double O(int shift)
{
   return(iOpen(NULL,PERIOD_M1,shift));
}

double H(int shift)
{
   return(iHigh(NULL,PERIOD_M1,shift));
}

double L(int shift)
{
   return(iLow(NULL,PERIOD_M1,shift));
}

double C(int shift)
{
   return(iClose(NULL,PERIOD_M1,shift));
}

datetime T(int shift)
{
   return(iTime(NULL,PERIOD_M1,shift));
}

int POWER(int shift)
{
   return(SUM(O(shift))+SUM(H(shift))+SUM(L(shift))+SUM(C(shift)));
}

void INIT(int shift)
{
   for(int i=0; i<PERIOD_D1; i++) POWER[i]=POWER(shift+i);
}
   
int MAX(int period)
{
   int S=0;
   for(int i=0; i<period; i++) if(POWER[i]>POWER[S]) S=i;
   return(S);
}

int MIN(int period)
{
   int S=0;
   for(int i=0; i<period; i++) if(POWER[i]<POWER[S]) S=i;
   return(S);
}

double EFFICIENCY(int S1,int S2)
{ 
   int P1=POWER(S1);
   int P2=POWER(S2);
   AVG=(P1+P2)/2;   
   string str1="\nPOWER (D1) max: "+P1;
   string str2="\nPOWER (D1) min: "+P2;   
   string str3="\nPOWER (D1) avg: "+AVG;   
   Comment(str1,str2,str3);
   return(MathMax(P1,P2)/MathMin(P1,P2));
}
   
void DRAW.D1()
{
   int S1=MAX[D1];
   int S2=MIN[D1];   
   double EF=EFFICIENCY(S1,S2);
   double P1=C(S1);
   double P2=C(S2);
   datetime T1=T(S1);
   datetime T2=T(S2);
   string VL1="VLINE1.D1";
   ObjectCreate(VL1,OBJ_VLINE,0,T1,0);
   ObjectSet(VL1,OBJPROP_COLOR,Lime);   
   string VL2="VLINE2.D1";
   ObjectCreate(VL2,OBJ_VLINE,0,T2,0);
   ObjectSet(VL2,OBJPROP_COLOR,Red);      
   string HL1="HLINE1.D1";
   ObjectCreate(HL1,OBJ_HLINE,0,0,P1);
   ObjectSet(HL1,OBJPROP_COLOR,Lime);      
   string HL2="HLINE2.D1";
   ObjectCreate(HL2,OBJ_HLINE,0,0,P2);
   ObjectSet(HL2,OBJPROP_COLOR,Red);      
   string TR1="TREND1.D1";
   ObjectCreate(TR1,OBJ_TREND,0,T1,P1,T2,P2);
   ObjectSet(TR1,OBJPROP_COLOR,Yellow);      
   string TR2="TREND2.D1";
   ObjectCreate(TR2,OBJ_TREND,0,T2,P2,T1,P1);
   ObjectSet(TR2,OBJPROP_COLOR,Yellow);      
   string REC="RECTANGLE.D1";     
   ObjectCreate(REC,OBJ_RECTANGLE,0,T1,P1,T2,P2);   
   double P0=MathAbs(P1-P2)*EF;
   double L1=MathMax(P1,P2)-P0;
   double L2=MathMin(P1,P2)+P0;
   string PL1="PLINE1.D1";
   ObjectCreate(PL1,OBJ_HLINE,0,0,L1);
   ObjectSet(PL1,OBJPROP_COLOR,Yellow);      
   string PL2="PLINE2.D1";
   ObjectCreate(PL2,OBJ_HLINE,0,0,L2);
   ObjectSet(PL2,OBJPROP_COLOR,Yellow);            
}

void DRAW.H4()
{
   int S1=MAX[H4];
   int S2=MIN[H4];
   double P1=C(S1);
   double P2=C(S2);
   datetime T1=T(S1);
   datetime T2=T(S2);
   string VL1="VLINE1.H4";
   ObjectCreate(VL1,OBJ_VLINE,0,T1,0);
   ObjectSet(VL1,OBJPROP_COLOR,Lime);   
   ObjectSet(VL1,OBJPROP_STYLE,STYLE_DASH);      
   string VL2="VLINE2.H4";
   ObjectCreate(VL2,OBJ_VLINE,0,T2,0);
   ObjectSet(VL2,OBJPROP_COLOR,Red);      
   ObjectSet(VL2,OBJPROP_STYLE,STYLE_DASH);         
   string HL1="HLINE1.H4";
   ObjectCreate(HL1,OBJ_HLINE,0,0,P1);
   ObjectSet(HL1,OBJPROP_COLOR,Lime);      
   ObjectSet(HL1,OBJPROP_STYLE,STYLE_DASH);         
   string HL2="HLINE2.H4";
   ObjectCreate(HL2,OBJ_HLINE,0,0,P2);
   ObjectSet(HL2,OBJPROP_COLOR,Red);      
   ObjectSet(HL2,OBJPROP_STYLE,STYLE_DASH);         
   string TR1="TREND1.H4";
   ObjectCreate(TR1,OBJ_TREND,0,T1,P1,T2,P2);
   ObjectSet(TR1,OBJPROP_COLOR,Yellow);      
   ObjectSet(TR1,OBJPROP_STYLE,STYLE_DASH);           
   string TR2="TREND2.H4";
   ObjectCreate(TR2,OBJ_TREND,0,T2,P2,T1,P1);
   ObjectSet(TR2,OBJPROP_COLOR,Yellow);        
   ObjectSet(TR2,OBJPROP_STYLE,STYLE_DASH);              
   string REC="RECTANGLE.H4";     
   ObjectCreate(REC,OBJ_RECTANGLE,0,T1,P1,T2,P2);   
}

void DRAW.H1()
{
   int S1=MAX[H1];
   int S2=MIN[H1];  
   double P1=C(S1);
   double P2=C(S2);
   datetime T1=T(S1);
   datetime T2=T(S2);
   string VL1="VLINE1.H1";
   ObjectCreate(VL1,OBJ_VLINE,0,T1,0);
   ObjectSet(VL1,OBJPROP_COLOR,Lime);   
   ObjectSet(VL1,OBJPROP_STYLE,STYLE_DOT);      
   string VL2="VLINE2.H1";
   ObjectCreate(VL2,OBJ_VLINE,0,T2,0);
   ObjectSet(VL2,OBJPROP_COLOR,Red);      
   ObjectSet(VL2,OBJPROP_STYLE,STYLE_DOT);         
   string HL1="HLINE1.H1";
   ObjectCreate(HL1,OBJ_HLINE,0,0,P1);
   ObjectSet(HL1,OBJPROP_COLOR,Lime);      
   ObjectSet(HL1,OBJPROP_STYLE,STYLE_DOT);         
   string HL2="HLINE2.H1";
   ObjectCreate(HL2,OBJ_HLINE,0,0,P2);
   ObjectSet(HL2,OBJPROP_COLOR,Red);      
   ObjectSet(HL2,OBJPROP_STYLE,STYLE_DOT);         
   string TR1="TREND1.H1";
   ObjectCreate(TR1,OBJ_TREND,0,T1,P1,T2,P2);
   ObjectSet(TR1,OBJPROP_COLOR,Yellow); 
   ObjectSet(TR1,OBJPROP_STYLE,STYLE_DOT);                   
   string TR2="TREND2.H1";
   ObjectCreate(TR2,OBJ_TREND,0,T2,P2,T1,P1);
   ObjectSet(TR2,OBJPROP_COLOR,Yellow);        
   ObjectSet(TR2,OBJPROP_STYLE,STYLE_DOT);                      
   string REC="RECTANGLE.H1";     
   ObjectCreate(REC,OBJ_RECTANGLE,0,T1,P1,T2,P2);   
}

int GAP(int index)
{
   int S1=MAX[index];
   int S2=MIN[index];
   double P1=C(S1);
   double P2=C(S2);
   if(S1<S2)
   if(P1>P2) return(+1);
   if(S1<S2)
   if(P1<P2) return(-2);
   if(S2<S1)
   if(P2<P1) return(-1);
   if(S2<S1)
   if(P2>P1) return(+2);   
   return(0);
}

bool INCREASE()
{
   int S1=MIN[D1];
   int S2=MIN[H4];
   int S3=MIN[H1];
   if(C(S1)<C(S2))
   if(C(S2)<C(S3)) return(true);
   return(false);
}

bool DECREASE()
{
   int S1=MAX[D1];
   int S2=MAX[H4];
   int S3=MAX[H1];
   if(C(S1)>C(S2))
   if(C(S2)>C(S3)) return(true);
   return(false);
}

int WAVE(int shift)
{
   Comment(shift);
   INIT(shift);  
   MAX[D1]=MAX(PERIOD_D1)+shift;
   MIN[D1]=MIN(PERIOD_D1)+shift;
   MAX[H4]=MAX(PERIOD_H4)+shift;
   MIN[H4]=MIN(PERIOD_H4)+shift;
   MAX[H1]=MAX(PERIOD_H1)+shift;
   MIN[H1]=MIN(PERIOD_H1)+shift;
   if(EQ(MAX[D1],MAX[H4]))
   if(EQ(MAX[H4],MAX[H1]))
   if(MR(MIN[D1],MIN[H4]))
   if(MR(MIN[H4],MIN[H1]))    
   if(EQ(GAP(D1),GAP(H4)))
   if(EQ(GAP(H4),GAP(H1))) if(INCREASE()) return(+1);
   if(EQ(MIN[D1],MIN[H4]))
   if(EQ(MIN[H4],MIN[H1]))
   if(MR(MAX[D1],MAX[H4]))
   if(MR(MAX[H4],MAX[H1]))       
   if(EQ(GAP(D1),GAP(H4)))
   if(EQ(GAP(H4),GAP(H1))) if(DECREASE()) return(-1);
   return(0);
}

bool LEFT(int shift)
{
   if(shift<MAX[D1])
   if(shift<MIN[D1]) return(true);
   return(false);
}

int SHIFT.1()
{
   int S=EMPTY;
   for(int i=1; LEFT(i); i++) if(EQ(POWER(i),AVG)) S=i;
   return(S);
}

int SHIFT.2()
{
   for(int i=1; LEFT(i); i++) if(EQ(POWER(i),AVG)) return(i);
   return(EMPTY);
}

void TRADE.1()
{
   if(NOT(Signal.1)) return;
   int S1=SHIFT.1();
   if(S1<0) return;
   datetime T1=T(S1);
   double P1=C(S1);
   string VT1="VTRADE1";
   ObjectCreate(VT1,OBJ_VLINE,0,T1,0);
   ObjectSet(VT1,OBJPROP_COLOR,Aqua);   
   string HT1="HTRADE1";
   ObjectCreate(HT1,OBJ_HLINE,0,0,P1);
   ObjectSet(HT1,OBJPROP_COLOR,Aqua);         
}

void TRADE.2()
{
   if(NOT(Signal.2)) return;
   int S2=SHIFT.2();
   if(S2<0) return;
   datetime T2=T(S2);
   double P2=C(S2);
   string VT2="VTRADE2";
   ObjectCreate(VT2,OBJ_VLINE,0,T2,0);
   ObjectSet(VT2,OBJPROP_COLOR,Aqua);   
   ObjectSet(VT2,OBJPROP_STYLE,STYLE_DASH);   
   string HT2="HTRADE2";
   ObjectCreate(HT2,OBJ_HLINE,0,0,P2);
   ObjectSet(HT2,OBJPROP_COLOR,Aqua);         
   ObjectSet(HT2,OBJPROP_STYLE,STYLE_DASH);      
}

bool DRAW(int shift)
{
   if(ZERO(WAVE(shift))) return(false);
   ObjectsDeleteAll();   
   DRAW.D1(); 
   DRAW.H4(); 
   DRAW.H1();
   TRADE.1();
   TRADE.2();
   return(true);
}

bool TICK()
{
   static datetime T;
   if(T<T(0)) 
      T=T(0); else return(false);
   return(true);      
}

void PAINT()
{
   for(int i=1; BARS(i); i++) if(DRAW(i)) break;
}

void start()
{
   if(TICK()) PAINT();
}


